from enum import Enum
import math
from faker import Faker
from fastapi import Body, FastAPI, Request, Response
from pydantic import BaseModel
import random
from fastapi.middleware.cors import CORSMiddleware

from settings import settings

app = FastAPI(
    title="Recommendation Engine",
    version="0.1.0",
)

app.add_middleware(CORSMiddleware, allow_origins=["*"], allow_methods=["*"])


@app.get("/")
async def hello():
    return "Hello world."


fake = Faker()


class BookingOptions(Enum):
    cheap = "Cheap"
    expensive = "Expensive"
    any = "Any"
    none = "None"


def get_hotel_option_from_sentence(sentence: str) -> str | None:
    hotel_option = BookingOptions.none

    lower_sentence = sentence.lower()

    hotel_half = next(
        (half for half in lower_sentence.split(" and ") if "hotel" in half), None
    )

    if hotel_half:
        if "cheap" in hotel_half:
            hotel_option = BookingOptions.cheap
        elif "expensive" in hotel_half:
            hotel_option = BookingOptions.expensive
        else:
            hotel_option = BookingOptions.any

    return find_hotel(hotel_option)


def get_car_option_from_sentence(sentence: str) -> str | None:
    car_option = BookingOptions.none

    lower_sentence = sentence.lower()

    car_half = next(
        (half for half in lower_sentence.split(" and ") if " car" in half), None
    )

    if car_half:
        if car_half and "cheap" in car_half:
            car_option = BookingOptions.cheap
        elif "expensive" in car_half:
            car_option = BookingOptions.expensive
        else:
            car_option = BookingOptions.any

    return find_car(car_option)


@app.post("/find")
async def find_from_sentence(
    request: Request, response: Response, sentence: str = Body(...)
):
    math.factorial(settings.speed.factorial)

    return {
        "hotel": get_hotel_option_from_sentence(sentence),
        "car": get_car_option_from_sentence(sentence),
    }


def find_hotel(hotel_option: BookingOptions) -> str | None:
    if hotel_option is BookingOptions.none:
        return None

    math.factorial(settings.speed.factorial)

    expensive_hotels = [
        "Park Hyatt",
        "Four Seasons",
        "The Ritz-Carlton",
        "InterContinental",
        "St. Regis",
        "Sofitel",
        "Shangri-La",
        "The Langham",
        "Crown Towers",
        "QT Hotels & Resorts",
        "COMO Hotels and Resorts",
        "The Darling",
        "The Star",
        "Crown Metropol",
        "Emporium Hotels",
    ]

    cheap_hotels = [
        "Ibis Budget",
        "Mantra Hotels",
        "Quest Apartment Hotels",
        "Travelodge Hotels",
        "ibis Styles",
        "Rydges Hotels and Resorts",
        "Vibe Hotels",
        "Metro Hotels",
        "Nightcap Hotels",
        "Econo Lodge",
        "Motel 6",
    ]

    if hotel_option == BookingOptions.cheap:
        return random.choice(cheap_hotels)
    elif hotel_option == BookingOptions.expensive:
        return random.choice(expensive_hotels)
    else:
        return random.choice(cheap_hotels + expensive_hotels)


def find_car(car_option: BookingOptions) -> str | None:
    if car_option is BookingOptions.none:
        return None

    math.factorial(settings.speed.factorial)

    expensive_cars = [
        "Mercedes-Benz S-Class",
        "BMW 7 Series",
        "Audi A8",
        "Lexus LS",
        "Porsche Panamera",
        "Tesla Model S",
        "Jaguar XJ",
        "Bentley Continental GT",
        "Rolls-Royce Phantom",
        "Maserati Quattroporte",
        "Aston Martin DB11",
        "Ferrari Portofino",
        "Lamborghini Huracán",
        "McLaren 720S",
        "Bugatti Chiron",
        "Maybach S-Class",
        "Genesis G90",
        "Bentley Flying Spur",
        "Koenigsegg Jesko",
        "Pagani Huayra",
    ]

    cheap_cars = [
        "Hyundai Accent",
        "Kia Rio",
        "Nissan Versa",
        "Ford Fiesta",
        "Toyota Yaris",
        "Chevrolet Spark",
        "Honda Fit",
        "Mitsubishi Mirage",
        "Volkswagen Jetta",
        "Subaru Impreza",
        "Mazda Mazda3",
        "FIAT 500",
        "Smart ForTwo",
        "Chevrolet Sonic",
        "Kia Soul",
        "Nissan Sentra",
        "Toyota Corolla",
        "Hyundai Elantra",
        "Ford Focus",
        "Honda Civic",
    ]

    if car_option == BookingOptions.cheap:
        return random.choice(cheap_cars)
    elif car_option == BookingOptions.expensive:
        return random.choice(expensive_cars)
    else:
        return random.choice(cheap_cars + expensive_cars)
