import { useMutation } from "@tanstack/react-query";
import { KeyboardEvent, useState } from "react";
import {
    Alert,
    Button,
    Card,
    Col,
    Container,
    InputGroup,
    Row,
} from "react-bootstrap";
import Form from "react-bootstrap/Form";
import "./App.scss";

import {
    faArrowRight,
    faArrowRotateLeft,
    faCar,
    faCheckCircle,
    faFire,
    faHotel,
    faSpinner,
} from "@fortawesome/free-solid-svg-icons";
import { FontAwesomeIcon } from "@fortawesome/react-fontawesome";
import { ReactQueryDevtools } from "@tanstack/react-query-devtools";
import axios from "axios";

const apiClient = axios.create({
    baseURL: "https://magic-agent-monolith.azurewebsites.net",
});

function App() {
    const [requestText, setRequestText] = useState("");
    const [recommendation, setRecommendation] = useState<any>({
        hotel: null,
        car: null,
    });
    const [confirmation, setConfirmation] = useState<any>({
        hotel: null,
        car: null,
    });
    const [apiBase, setAPIBase] = useState<string>("http://20.70.85.23");

    const mutateFind = useMutation({
        mutationFn: (sentence: string) => {
            return apiClient.post(`${apiBase}/find`, sentence);
        },
    });

    const mutateBook = useMutation({
        mutationFn: (payload: any) => {
            return apiClient.post(`${apiBase}/book`, JSON.stringify(payload), {
                headers: {
                    "Content-Type": "application/json",
                },
            });
        },
    });

    const handleSubmit = async (sentence: string) => {
        if (sentence.length === 0) {
            return;
        }
        let result = await mutateFind.mutateAsync(requestText);
        setRecommendation(result.data);
    };

    const handleBook = async () => {
        let result = await mutateBook.mutateAsync(recommendation);
        setConfirmation(result.data);
    };

    return (
        <>
            <Container style={{ position: "absolute" }}>
                <center>
                    <Form.Select
                        className="mt-3"
                        onChange={(e) => setAPIBase(e.target.value)}
                        style={{ maxWidth: "250px" }}
                    >
                        <option value="http://20.70.85.23">Monolith</option>
                        <option value="http://20.70.86.75">Microservices</option>
                        <option value="http://localhost:8000">Local</option>
                    </Form.Select>
                    API Endpoint: <code>{apiBase}</code>
                </center>
            </Container>
            <div className="App">
                <Container className="app-container">
                    <h1>
                        <center>Welcome to Steven's Magic Travel Agent</center>
                    </h1>
                    <InputGroup className="mt-4">
                        <Form.Control
                            size="lg"
                            placeholder="What would you like to book?"
                            value={requestText}
                            onChange={(e) =>
                                setRequestText(e.target.value)
                            }
                            onKeyDown={(target: KeyboardEvent<HTMLElement>) => {
                                if (target.key === "Enter") {
                                    handleSubmit(requestText);
                                }
                            }}
                            disabled={
                                mutateFind.isLoading ||
                                mutateBook.isLoading ||
                                mutateBook.isSuccess
                            }
                        />
                        <Button
                            variant="outline-light"
                            onClick={() => handleSubmit(requestText)}
                            disabled={
                                requestText.length === 0 ||
                                mutateFind.isLoading ||
                                mutateBook.isLoading ||
                                mutateBook.isSuccess
                            }
                        >
                            <FontAwesomeIcon
                                icon={
                                    mutateFind.isLoading
                                        ? faSpinner
                                        : faArrowRight
                                }
                                size="lg"
                                spin={mutateFind.isLoading}
                            />
                        </Button>
                    </InputGroup>
                    {recommendation &&
                        (recommendation.hotel || recommendation.car) && (
                            <>
                                <Row className="mt-4">
                                    {recommendation.hotel && (
                                        <Col>
                                            <Card>
                                                <Card.Header>
                                                    <FontAwesomeIcon
                                                        icon={faHotel}
                                                        size="lg"
                                                    />
                                                    &nbsp; Hotel Recommendation
                                                </Card.Header>
                                                <Card.Body>
                                                    <p className="name-text">
                                                        {recommendation.hotel}
                                                    </p>
                                                </Card.Body>
                                            </Card>
                                        </Col>
                                    )}
                                    {recommendation.car && (
                                        <Col>
                                            <Card>
                                                <Card.Header>
                                                    <FontAwesomeIcon
                                                        icon={faCar}
                                                        size="lg"
                                                    />
                                                    &nbsp; Car Recommendation
                                                </Card.Header>
                                                <Card.Body>
                                                    <p className="name-text">
                                                        {recommendation.car}
                                                    </p>
                                                </Card.Body>
                                            </Card>
                                        </Col>
                                    )}
                                </Row>
                                <div className="mt-4">
                                    <center>
                                        <Button
                                            className="book-button"
                                            onClick={handleBook}
                                            disabled={
                                                mutateBook.isLoading ||
                                                mutateBook.isSuccess
                                            }
                                        >
                                            {mutateBook.isLoading ? (
                                                <FontAwesomeIcon
                                                    icon={faSpinner}
                                                    spin
                                                />
                                            ) : (
                                                "Book"
                                            )}
                                        </Button>
                                        <Button className="reset-book-button" variant="outline-light" onClick={() => mutateBook.reset()} disabled={!mutateBook.isSuccess}>
                                            <FontAwesomeIcon icon={faArrowRotateLeft} />
                                        </Button>
                                    </center>
                                </div>
                                {mutateBook.isSuccess && (
                                    <Row className="mt-4">
                                        <Alert variant="success">
                                            <FontAwesomeIcon
                                                icon={faFire}
                                                size="lg"
                                            />
                                            &nbsp; All booked! Find the details
                                            below:
                                        </Alert>
                                        {confirmation.hotel &&
                                            confirmation.hotel.code && (
                                                <Col>
                                                    <Card>
                                                        <Card.Header>
                                                            <FontAwesomeIcon
                                                                icon={
                                                                    faCheckCircle
                                                                }
                                                                size="lg"
                                                            />
                                                            &nbsp; Hotel
                                                            Confirmation
                                                        </Card.Header>
                                                        <Card.Body className="confirmation-card-body">
                                                            <code className="confirmation-code">
                                                                {
                                                                    confirmation
                                                                        .hotel
                                                                        .name
                                                                }
                                                            </code>
                                                            <br />
                                                            <code className="confirmation-code">
                                                                {
                                                                    confirmation
                                                                        .hotel
                                                                        .code
                                                                }
                                                            </code>
                                                        </Card.Body>
                                                    </Card>
                                                </Col>
                                            )}
                                        {confirmation.car &&
                                            confirmation.car.code && (
                                                <Col>
                                                    <Card>
                                                        <Card.Header>
                                                            <FontAwesomeIcon
                                                                icon={
                                                                    faCheckCircle
                                                                }
                                                                size="lg"
                                                            />
                                                            &nbsp; Car
                                                            Confirmation
                                                        </Card.Header>
                                                        <Card.Body className="confirmation-card-body">
                                                            <code className="confirmation-code">
                                                                {
                                                                    confirmation
                                                                        .car
                                                                        .name
                                                                }
                                                            </code>
                                                            <br />
                                                            <code className="confirmation-code">
                                                                {
                                                                    confirmation
                                                                        .car
                                                                        .code
                                                                }
                                                            </code>
                                                        </Card.Body>
                                                    </Card>
                                                </Col>
                                            )}
                                    </Row>
                                )}
                            </>
                        )}
                </Container>
            </div>
            <ReactQueryDevtools initialIsOpen={false} />
        </>
    );
}

export default App;
